@@ -10,7 +10,7 @@ use warnings;
 use strict;
 
 package Curses::Toolkit::Role::Focusable;
-our $VERSION = '0.093060';
+our $VERSION = '0.100320';
 
 
 # ABSTRACT: This role implements the fact that a widget can have focus
@@ -44,18 +44,20 @@ sub set_focus {
 	if ($self->can('get_window')) {
 		my $window = $self->get_window();
 		if (defined $window) {
-			my $root_window = $window->get_root_window();
-			if (defined $root_window) {
-				if ($focus) {
-					use Curses::Toolkit::Event::Focus::In;
-					my $event_focus_in = Curses::Toolkit::Event::Focus::In->new( root_window => $root_window );
-					$root_window->dispatch_event($event_focus_in, $self, 1);
-					$window->set_focused_widget($self);
-				} else {
-					use Curses::Toolkit::Event::Focus::Out;
-					my $event_focus_out = Curses::Toolkit::Event::Focus::Out->new( root_window => $root_window );
-					$root_window->dispatch_event($event_focus_out, $self, 1);
-				}
+			if ($focus) {
+				use Curses::Toolkit::Event::Focus::In;
+				# restrict the event to the widget
+				my $event_focus_in = Curses::Toolkit::Event::Focus::In
+				                     ->new
+									 ->enable_restriction;
+				$self->fire_event($event_focus_in, $self);
+				$window->set_focused_widget($self);
+			} else {
+				use Curses::Toolkit::Event::Focus::Out;
+				my $event_focus_out = Curses::Toolkit::Event::Focus::Out
+				                      ->new
+									  ->enable_restriction;
+				$self->fire_event($event_focus_out, $self);
 			}
 		}
 	}
@@ -84,7 +86,7 @@ Curses::Toolkit::Role::Focusable - This role implements the fact that a widget c
 
 =head1 VERSION
 
-version 0.093060
+version 0.100320
 
 =head1 DESCRIPTION
 
