@@ -10,7 +10,7 @@ use warnings;
 use strict;
 
 package Curses::Toolkit::Theme::Default;
-our $VERSION = '0.093060';
+our $VERSION = '0.100320';
 
 
 # ABSTRACT: default widget theme
@@ -44,12 +44,20 @@ sub _get_default_properties {
 					  # inherited from Border
 					  border_width => 1,
 					},
+					'Curses::Toolkit::Widget::Button' => {
+					  # inherited from Border
+					  border_width => 0,
+					  left_enclosing => '< ',
+					  right_enclosing => ' >',
+					},
 #  					'Curses::Toolkit::Widget::Paned' => {
 #  					  gutter_size => 1,
 #  					},
-# 					'Curses::Toolkit::Widget::Entry' => {
-# 					  default_width => 20,
-# 					},
+ 					'Curses::Toolkit::Widget::Entry' => {
+ 					  default_width => 3,
+					  left_enclosing => '[',
+					  right_enclosing => ']',
+ 					},
 				  );
 	return $default{$class_name} || {};
 }
@@ -87,9 +95,9 @@ sub RESIZE_NORMAL  { }
 sub RESIZE_FOCUSED { shift->_attron(A_REVERSE) }
 sub RESIZE_CLICKED { shift->_attron(A_BOLD) }
 
-sub BLANK_NORMAL  { shift->_attrset() }
-sub BLANK_FOCUSED { shift->_attrset() }
-sub BLANK_CLICKED { shift->_attrset() }
+sub BLANK_NORMAL  { shift->_set_colors('white', 'black') }
+sub BLANK_FOCUSED { shift->_set_colors('white', 'black') }
+sub BLANK_CLICKED { shift->_set_colors('white', 'black') }
 
 sub draw_hline {
 	my ($self, $x1, $y1, $width, $attr) = @_;
@@ -146,10 +154,19 @@ sub draw_corner_lr {
 sub draw_string {
 	my ($self, $x1, $y1, $text, $attr) = @_;
 	$self->get_widget->is_visible() or return;
-	my $c = $self->restrict_to_shape(x1 => $x1, y1 => $y1, width => length($text), height => 1) or return;
-	$text = substr($text, $c->x1()-$x1, $c->width());
-	defined $text && length $text or return;
-	$self->curses($attr)->addstr($c->y1(), $c->x1(), $text);
+
+	use Curses::Toolkit::Object::MarkupString;
+	ref $text or
+	  $text = Curses::Toolkit::Object::MarkupString->new($text);
+
+	my $c = $self->restrict_to_shape(x1 => $x1, y1 => $y1, width => $text->stripped_length(), height => 1) or return;
+
+	my $start = $c->x1() - $x1;
+	my $end = $c->x1() - $x1 + $c->width();
+	my $width = $end - $start;
+	$text = $text->substring($start, $width);
+	$text->stripped_length() or return;
+	$self->_addstr_with_tags($attr, $c->x1(), $c->y1(), $text);
 	return $self;
 }
 
@@ -157,7 +174,10 @@ sub draw_title {
 	my ($self, $x1, $y1, $text, $attr) = @_;
 	$self->get_widget->is_visible() or return;
 	my $c = $self->restrict_to_shape(x1 => $x1, y1 => $y1, width => length($text), height => 1) or return;
-	$text = substr($text, $c->x1()-$x1, $c->width());
+
+	$c->x1() - $x1 < length $text
+	  or return;
+	$text = substr($text, $c->x1() - $x1, $c->width());
 	defined $text && length $text or return;
 	$self->curses($attr)->addstr($c->y1(), $c->x1(), $text);
 	return $self;
@@ -198,7 +218,7 @@ Curses::Toolkit::Theme::Default - default widget theme
 
 =head1 VERSION
 
-version 0.093060
+version 0.100320
 
 =head1 DESCRIPTION
 
