@@ -10,7 +10,7 @@ use warnings;
 use strict;
 
 package Curses::Toolkit::Widget::Border;
-our $VERSION = '0.093060';
+our $VERSION = '0.100320';
 
 
 # ABSTRACT: a border widget
@@ -111,7 +111,13 @@ Curses::Toolkit::Widget::Border - a border widget
 
 =head1 VERSION
 
-version 0.093060
+version 0.100320
+
+=head1 Appearence
+
+  +----------+
+  |          |
+  +----------+
 
 =head1 DESCRIPTION
 
@@ -152,19 +158,19 @@ needed to properly display itself
 
 To set/get a theme properties, you should do :
 
-$border->set_theme_property(property_name => $property_value);
-$value = $border->get_theme_property('property_name')
+  $border->set_theme_property(property_name => $property_value);
+  $value = $border->get_theme_property('property_name');
 
 Here is the list of properties related to the border, that can be changed in
 the associated theme. See the Curses::Toolkit::Theme class used for the default
 (default class to look at is Curses::Toolkit::Theme::Default)
 
 Don't forget to look at properties from the parent class, as these are also
-inherited of !
+inherited from !
 
 =head2 border_width
 
-Sets the with of the window border. If not set, the window border will be invisible
+Sets the width of the border. If not set, the border will be invisible
 
 
 
