@@ -10,7 +10,7 @@ use warnings;
 use strict;
 
 package Curses::Toolkit::Widget::Label;
-our $VERSION = '0.093060';
+our $VERSION = '0.100320';
 
 
 # ABSTRACT: a container with two panes arranged horizontally
@@ -19,6 +19,7 @@ use parent qw(Curses::Toolkit::Widget);
 
 use Params::Validate qw(:all);
 use List::Util qw(min max);
+use Curses::Toolkit::Object::MarkupString;
 
 
 sub new {
@@ -37,6 +38,7 @@ sub set_text {
 	
 	my ($text) = validate_pos( @_, { type => SCALAR } );
 	$self->{text} = $text;
+	$self->{_markup_string} = Curses::Toolkit::Object::MarkupString->new($text);
 	$self->needs_redraw();
 	return $self;
 
@@ -94,18 +96,18 @@ sub draw {
 	my ($self) = @_;
 	my $theme = $self->get_theme();
 	my $c = $self->get_coordinates();
-	my $text = $self->get_text();
+	my $text = $self->{_markup_string}->stripped();
 
 	my $justify = $self->get_justify();
 
 	my $wrap_method = $self->get_wrap_method();
 
-	my @text = _textwrap($text, $c->width());
+	my @text = _textwrap($self->{_markup_string}, $c->width());
 
 	foreach my $y ( 0..min($#text, $c->height() - 1) ) {
 		my $t = $text[$y];
-		$t =~ s/^\s+//g;
-		$t =~ s/\s+$//g;
+		$t->search_replace('^\s+', '');
+		$t->search_replace('\s+$', '');
 		if ($justify eq 'left') {
 			$theme->draw_string($c->x1(), $c->y1() + $y, $t);
 		}
@@ -129,40 +131,49 @@ sub _textwrap {
   my (@tmp, @rv, $p);
 
   # Early exit if no text was passed
-  return unless (defined $text && length($text));
+  return unless (defined $text && $text->stripped_length() );
 
   # Split the text into paragraphs, but preserve the terminating newline
-  @tmp = split(/\n/, $text);
-  foreach (@tmp) { $_ .= "\n" };
-  chomp($tmp[$#tmp]) unless $text =~ /\n$/;
+  @tmp = $text->split_string("\n");
+  foreach my $t (@tmp) {
+	  $t->append("\n");
+  }
+
+  $tmp[-1]->chomp_string() unless $text->stripped() =~ /\n$/;
 
   # Split each paragraph into lines, according to whitespace
   for $p (@tmp) {
 
     # Snag lines that meet column limits (not counting newlines
     # as a character)
-    if (length($p) <= $columns || (length($p) - 1 <= $columns &&
-      $p =~ /\n$/s)) {
+    if ($p->stripped_length() <= $columns || ($p->stripped_length() - 1 <= $columns &&
+      $p->stripped() =~ /\n$/s)) {
       push(@rv, $p);
       next;
     }
 
     # Split the line
-    while (length($p) > $columns) {
-      if (substr($p, 0, $columns) =~ /^(.+\s)(\S+)$/) {
-        push(@rv, $1);
-        $p = $2 . substr($p, $columns);
+    while ($p->stripped_length() > $columns) {
+      if ($p->substring(0, $columns)->stripped() =~ /^(.+\s)(\S+)$/) {
+		  my ($v1, $v2) = ($1, $2);
+		  push(@rv, $p->substring(0, length($v1)) );
+		  my $l = $p->stripped_length();
+		  my $m1 = $p->substring(length($v1), length($v2));
+		  my $m2 = $p->substring($columns, $l - $columns );
+		  $m1->append($m2);
+		  $p = $m1;
       } else {
-        push(@rv, substr($p, 0, $columns));
-        substr($p, 0, $columns) = '';
+		  push(@rv, $p->substring(0, $columns) );
+		  $p = $p->substring($columns, $p->stripped_length() - $columns );
       }
     }
+
     push(@rv, $p);
   }
 
-  if ($text =~ /\S\n(\n+)/) {
-    $p = length($1);
-    foreach (1..$p) { push(@rv, "\n") };
+  if ($text->stripped() =~ /\S\n(\n+)/) {
+    my $l = length($1);
+    foreach (1..$l) { push(@rv, Curses::Toolkit::Object::MarkupString->new("\n")) };
   }
 
   return @rv;
@@ -181,7 +192,7 @@ sub get_minimum_space {
 
 	my $minimum_space = $available_space->clone();
 	my $wrap_mode = $self->get_wrap_mode();
-	my $text = $self->get_text();
+	my $text = $self->{_markup_string}->stripped();
 	if ($wrap_mode eq 'never') {
 		$text =~ s/\n(\s)/$1/g;
 		$text =~ s/\n/ /g;
@@ -192,9 +203,9 @@ sub get_minimum_space {
 	} elsif ($wrap_mode eq 'active') {
 		my $width = 1;
 		while (1) {
-			my @text = _textwrap($self->get_text(), $width);
-			if ($width >= length($self->get_text())) {
-				$minimum_space->set( x2 => $minimum_space->x1() + length($self->get_text()) + 1,
+			my @text = _textwrap($self->{_markup_string}, $width);
+			if ($width >= $self->{_markup_string}->stripped_length()) {
+				$minimum_space->set( x2 => $minimum_space->x1() + $self->{_markup_string}->stripped_length() + 1,
 									 y2 => $minimum_space->y1() + 1 );
 				last;
 			}
@@ -202,15 +213,15 @@ sub get_minimum_space {
 				$width++;
 				next;
 			}
-			$minimum_space->set( x2 => $minimum_space->x1() + max(map { length } @text ) + 1,
+			$minimum_space->set( x2 => $minimum_space->x1() + max(map { $_->stripped_length() } @text ) + 1,
 								 y2 => $minimum_space->y1() + scalar(@text) );
 			last;
 		}
 		return $minimum_space;
 	} elsif ($wrap_mode eq 'lazy') {
-		my @text = _textwrap($self->get_text(), max($available_space->width(), 1));
+		my @text = _textwrap($self->{_markup_string}, max($available_space->width(), 1));
 		$minimum_space->set( y2 => $minimum_space->y1() + scalar(@text) );
-		$minimum_space->set( x2 => $minimum_space->x1() + max(map { length } @text ) );
+		$minimum_space->set( x2 => $minimum_space->x1() + max(map { $_->stripped_length() } @text ) );
 		return $minimum_space;
 	}
 	die;
@@ -227,11 +238,93 @@ Curses::Toolkit::Widget::Label - a container with two panes arranged horizontall
 
 =head1 VERSION
 
-version 0.093060
+version 0.100320
 
 =head1 DESCRIPTION
 
-This widget consists of a text label
+This widget consists of a text label. This widget is more powerful than it
+seems : it supports line wrapping, and color, bold, underline, etc.
+
+=head1 MARKUPS SUPPORT
+
+To be able to have more than simple text, the Label widget supports markup tags
+in its text, for example :
+
+  'foo <u>underlined bar</u> <span fgcolor="blue"> blue text <span
+   bgcolor="red"> blue on red </span> normal on red </span> <b>bold</b>.'
+
+=over 
+
+=item <u>
+
+  <u>underlined string</u>
+
+The <u> tag makes the enclosing text underlined
+
+=item <b>
+
+  <b>bold string</b>
+
+The <b> tag makes the enclosing text bold
+
+=item <span>
+
+The <span> tag allows more attributes to be set. Attributes can of course be combined :
+
+  <span wight="blink" fgcolor="black" bgcolor="red">Warning text!</span>
+
+There is the list of attributes :
+
+=over 
+
+=item weight
+
+  <span weight="reverse">some reverse string</span>
+
+Specifies display attributes. Weight values can be :
+
+  normal : force some text back to normal
+  standout : enable standout property
+  underline : enable underline property
+  blink : enable blink property
+  dim : enable dim property
+  bold : enable bold property
+
+Somme properties may be unsupported on your terminal.
+
+=item fgcolor
+
+  <span fgcolor="blue">some blue text</span>
+
+Change the foreground color. values can be :
+
+  black
+  red
+  green
+  yellow
+  blue
+  magenta
+  cyan
+  white
+
+=item bgcolor
+
+  <span bgcolor="red">some red background text</span>
+
+Change the foreground color. values can be :
+
+  black
+  red
+  green
+  yellow
+  blue
+  magenta
+  cyan
+  white
+
+=back 
+
+=back 
 
 =head1 CONSTRUCTOR
 
@@ -246,7 +339,8 @@ This widget consists of a text label
 
 =head2 set_text
 
-Set the text of the label
+Set the text of the label. The text can be either normal text, or text with
+markups, to display colors, bold, underline, etc., see Markup Support above
 
   input  : the text
   output : the label object
